@@ -10,7 +10,6 @@ This module implements differential operators on Cartesian grids
    make_vector_gradient
    make_vector_laplace
    make_tensor_divergence
-   make_operator
    
    
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>   
@@ -169,6 +168,88 @@ def _get_laplace_matrix(bcs: Boundaries):
 
 
 
+def _make_derivative(bcs: Boundaries, axis: int = 0, method: str = 'central') \
+        -> Callable:
+    """ make a derivative operator along a single axis using numba compilation
+    
+    Args:
+        bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
+            {ARG_BOUNDARIES_INSTANCE}
+        axis (int):
+            The axis along which the derivative will be taken
+        method (str):
+            The method for calculating the derivative. Possible values are
+            'central', 'forward', and 'backward'.
+        
+    Returns:
+        A function that can be applied to an array of values. The result will be
+        an array of the same shape containing the actual derivatives at the grid
+        points.
+    """
+    if method not in {'central', 'forward', 'backward'}:
+        raise ValueError(f'Unknown derivative type `{method}`')
+    
+    shape = bcs.grid.shape
+    dim = len(shape)
+    dx = bcs.grid.discretization[axis]
+    region = bcs[axis].make_region_evaluator()
+        
+    if dim == 1:
+        @jit_allocate_out(out_shape=shape)
+        def diff(arr, out=None):
+            """ calculate derivative of 1d array `arr` """
+            for i in range(shape[0]):
+                arr_l, arr_m, arr_h = region(arr, (i,))
+                if method == 'central':
+                    out[i] = (arr_h - arr_l) * 0.5 / dx
+                elif method == 'forward':
+                    out[i] = (arr_h - arr_m) / dx
+                elif method == 'backward':
+                    out[i] = (arr_m - arr_l) / dx
+                    
+            return out       
+        
+    elif dim == 2:
+        @jit_allocate_out(out_shape=shape)
+        def diff(arr, out=None):
+            """ calculate derivative of 2d array `arr` """
+            for i in range(shape[0]):
+                for j in range(shape[1]):
+                    arr_l, arr_m, arr_h = region(arr, (i, j))
+                    if method == 'central':
+                        out[i, j] = (arr_h - arr_l) * 0.5 / dx
+                    elif method == 'forward':
+                        out[i, j] = (arr_h - arr_m) / dx
+                    elif method == 'backward':
+                        out[i, j] = (arr_m - arr_l) / dx
+                    
+            return out       
+        
+    elif dim == 3:
+        @jit_allocate_out(out_shape=shape)
+        def diff(arr, out=None):
+            """ calculate derivative of 3d array `arr` """
+            for i in range(shape[0]):
+                for j in range(shape[1]):
+                    for k in range(shape[2]):
+                        arr_l, arr_m, arr_h = region(arr, (i, j, k))
+                        if method == 'central':
+                            out[i, j, k] = (arr_h - arr_l) * 0.5 / dx
+                        elif method == 'forward':
+                            out[i, j, k] = (arr_h - arr_m) / dx
+                        elif method == 'backward':
+                            out[i, j, k] = (arr_m - arr_l) / dx
+                    
+            return out     
+        
+    else:
+        raise NotImplementedError('Numba derivative operator not implemented '
+                                  f'for {dim:d} dimensions')
+        
+    return diff  # type: ignore    
+
+
+
 @fill_in_docstring
 def _make_laplace_scipy_nd(bcs: Boundaries) -> Callable:
     """ make a laplace operator using the scipy module
@@ -225,7 +306,7 @@ def _make_laplace_numba_1d(bcs: Boundaries) -> Callable:
     """
     dim_x = bcs.grid.shape[0]
     scale = bcs.grid.discretization[0]**-2
-    region_x = bcs[0].get_region_evaluator()
+    region_x = bcs[0].make_region_evaluator()
     
     @jit_allocate_out
     def laplace(arr, out=None):
@@ -254,8 +335,8 @@ def _make_laplace_numba_2d(bcs: Boundaries) -> Callable:
     dim_x, dim_y = bcs.grid.shape
     scale_x, scale_y = bcs.grid.discretization**-2
     
-    region_x = bcs[0].get_region_evaluator()
-    region_y = bcs[1].get_region_evaluator()
+    region_x = bcs[0].make_region_evaluator()
+    region_y = bcs[1].make_region_evaluator()
     
     # use parallel processing for large enough arrays 
     parallel = (dim_x * dim_y >= PARALLELIZATION_THRESHOLD_2D**2)
@@ -314,9 +395,9 @@ def _make_laplace_numba_3d(bcs: Boundaries) -> Callable:
     dim_x, dim_y, dim_z = bcs.grid.shape 
     scale_x, scale_y, scale_z = bcs.grid.discretization**-2
     
-    region_x = bcs[0].get_region_evaluator()
-    region_y = bcs[1].get_region_evaluator()
-    region_z = bcs[2].get_region_evaluator()
+    region_x = bcs[0].make_region_evaluator()
+    region_y = bcs[1].make_region_evaluator()
+    region_z = bcs[2].make_region_evaluator()
     
     # use parallel processing for large enough arrays 
     parallel = (dim_x * dim_y * dim_z >= PARALLELIZATION_THRESHOLD_3D**3)
@@ -389,6 +470,11 @@ def make_laplace(bcs: Boundaries, method: str = 'auto') -> Callable:
     return laplace
 
 
+# register operators with the grid class
+CartesianGridBase.register_operator('laplace', make_laplace,
+                                    rank_in=0, rank_out=0)
+
+
 
 @fill_in_docstring
 def _make_gradient_scipy_nd(bcs: Boundaries) -> Callable:
@@ -431,7 +517,7 @@ def _make_gradient_numba_1d(bcs: Boundaries) -> Callable:
     """
     dim_x = bcs.grid.shape[0]
     scale = 0.5 / bcs.grid.discretization[0]
-    region_x = bcs[0].get_region_evaluator()
+    region_x = bcs[0].make_region_evaluator()
     
     @jit_allocate_out(out_shape=(1, dim_x))
     def gradient(arr, out=None):
@@ -460,8 +546,8 @@ def _make_gradient_numba_2d(bcs: Boundaries) -> Callable:
     dim_x, dim_y = bcs.grid.shape
     scale_x, scale_y = 0.5 / bcs.grid.discretization
     
-    region_x = bcs[0].get_region_evaluator()
-    region_y = bcs[1].get_region_evaluator()
+    region_x = bcs[0].make_region_evaluator()
+    region_y = bcs[1].make_region_evaluator()
 
     # use parallel processing for large enough arrays 
     parallel = (dim_x * dim_y >= PARALLELIZATION_THRESHOLD_2D**2)
@@ -497,9 +583,9 @@ def _make_gradient_numba_3d(bcs: Boundaries) -> Callable:
     dim_x, dim_y, dim_z = bcs.grid.shape
     scale_x, scale_y, scale_z = 0.5 / bcs.grid.discretization
     
-    region_x = bcs[0].get_region_evaluator()
-    region_y = bcs[1].get_region_evaluator()
-    region_z = bcs[2].get_region_evaluator()
+    region_x = bcs[0].make_region_evaluator()
+    region_y = bcs[1].make_region_evaluator()
+    region_z = bcs[2].make_region_evaluator()
     
     # use parallel processing for large enough arrays 
     parallel = (dim_x * dim_y * dim_z >= PARALLELIZATION_THRESHOLD_3D**3)
@@ -567,6 +653,10 @@ def make_gradient(bcs: Boundaries, method: str = 'auto') -> Callable:
     return gradient
 
 
+# register operators with the grid class
+CartesianGridBase.register_operator('gradient', make_gradient,
+                                    rank_in=0, rank_out=1)
+
 
 
 @fill_in_docstring
@@ -618,7 +708,7 @@ def _make_divergence_numba_1d(bcs: Boundaries) -> Callable:
     """
     dim_x = bcs.grid.shape[0]
     scale = 0.5 / bcs.grid.discretization[0]
-    region_x = bcs[0].get_region_evaluator()
+    region_x = bcs[0].make_region_evaluator()
     
     @jit_allocate_out(out_shape=(dim_x,))
     def divergence(arr, out=None):
@@ -647,8 +737,8 @@ def _make_divergence_numba_2d(bcs: Boundaries) -> Callable:
     dim_x, dim_y = bcs.grid.shape
     scale_x, scale_y = 0.5 / bcs.grid.discretization
     
-    region_x = bcs[0].get_region_evaluator()
-    region_y = bcs[1].get_region_evaluator()
+    region_x = bcs[0].make_region_evaluator()
+    region_y = bcs[1].make_region_evaluator()
 
     # use parallel processing for large enough arrays 
     parallel = (dim_x * dim_y >= PARALLELIZATION_THRESHOLD_2D**2)
@@ -685,9 +775,9 @@ def _make_divergence_numba_3d(bcs: Boundaries) -> Callable:
     dim_x, dim_y, dim_z = bcs.grid.shape
     scale_x, scale_y, scale_z = 0.5 / bcs.grid.discretization
     
-    region_x = bcs[0].get_region_evaluator()
-    region_y = bcs[1].get_region_evaluator()
-    region_z = bcs[2].get_region_evaluator()
+    region_x = bcs[0].make_region_evaluator()
+    region_y = bcs[1].make_region_evaluator()
+    region_z = bcs[2].make_region_evaluator()
     
     # use parallel processing for large enough arrays 
     parallel = (dim_x * dim_y * dim_z >= PARALLELIZATION_THRESHOLD_3D**3)
@@ -756,6 +846,11 @@ def make_divergence(bcs: Boundaries, method: str = 'auto') -> Callable:
     return divergence
 
 
+# register operators with the grid class
+CartesianGridBase.register_operator('divergence', make_divergence,
+                                    rank_in=1, rank_out=0)
+
+
 
 @fill_in_docstring
 def _make_vector_gradient_scipy_nd(bcs: Boundaries) -> Callable:
@@ -908,6 +1003,12 @@ def make_vector_gradient(bcs: Boundaries, method: str = 'auto') -> Callable:
     return gradient
 
 
+# register operators with the grid class
+CartesianGridBase.register_operator('vector_gradient', make_vector_gradient,
+                                    rank_in=1, rank_out=2)
+
+
+
 @fill_in_docstring
 def _make_vector_laplace_scipy_nd(bcs: Boundaries) -> Callable:
     """ make a vector Laplacian using the scipy module
@@ -1055,6 +1156,11 @@ def make_vector_laplace(bcs: Boundaries, method: str = 'auto') -> Callable:
     return gradient
 
 
+# register operators with the grid class
+CartesianGridBase.register_operator('vector_laplace', make_vector_laplace,
+                                    rank_in=1, rank_out=1)
+
+
 
 @fill_in_docstring
 def _make_tensor_divergence_scipy_nd(bcs: Boundaries) -> Callable:
@@ -1210,6 +1316,11 @@ def make_tensor_divergence(bcs: Boundaries, method: str = 'auto') -> Callable:
     return func
 
 
+# register operators with the grid class
+CartesianGridBase.register_operator('tensor_divergence', make_tensor_divergence,
+                                    rank_in=2, rank_out=1)
+
+
 
 @fill_in_docstring
 def make_poisson_solver(bcs: Boundaries, method: str = 'auto') -> Callable:
@@ -1229,56 +1340,13 @@ def make_poisson_solver(bcs: Boundaries, method: str = 'auto') -> Callable:
     return make_general_poisson_solver(matrix, vector, method)
 
 
-
-@fill_in_docstring
-def make_operator(op: str, bcs: Boundaries, method: str = 'auto') -> Callable:
-    """ return a discretized operator for a Cartesian grid
-    
-    Args:
-        op (str): Identifier for the operator. Some examples are 'laplace',
-            'gradient', or 'divergence'.
-        bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            {ARG_BOUNDARIES_INSTANCE}
-        method (str): The method for implementing the operator
-        
-    Returns:
-        A function that takes the discretized data as an input and returns
-        the data to which the operator `op` has been applied. This function
-        optionally supports a second argument, which provides allocated
-        memory for the output.
-    """
-    if op == 'laplace' or op == 'laplacian':
-        return make_laplace(bcs, method)
-    elif op == 'gradient':
-        return make_gradient(bcs, method)
-    elif op == 'divergence':
-        return make_divergence(bcs, method)
-    elif op == 'vector_gradient':
-        return make_vector_gradient(bcs, method) 
-    elif op == 'vector_laplace' or op == 'vector_laplacian':
-        return make_vector_laplace(bcs, method)
-    elif op == 'tensor_divergence':
-        return make_tensor_divergence(bcs, method)
-    elif op == 'poisson_solver' or op == 'solve_poisson' or op == 'poisson':
-        matrix, vector = _get_laplace_matrix(bcs)
-        return make_general_poisson_solver(matrix, vector, method)
-    else:
-        raise NotImplementedError(f'Operator `{op}` is not defined for '
-                                  'Cartesian grids')
-
-
-# register all operators with the grid class
-CartesianGridBase.register_operator('laplace', make_laplace)
-CartesianGridBase.register_operator('gradient', make_gradient)
-CartesianGridBase.register_operator('divergence', make_divergence)
-CartesianGridBase.register_operator('vector_gradient', make_vector_gradient)
-CartesianGridBase.register_operator('vector_laplace', make_vector_laplace)
-CartesianGridBase.register_operator('tensor_divergence', make_tensor_divergence)
-CartesianGridBase.register_operator('poisson_solver', make_poisson_solver)
+# register operators with the grid class
+CartesianGridBase.register_operator('poisson_solver', make_poisson_solver,
+                                    rank_in=0, rank_out=0)
 
 
 
 __all__ = ["make_laplace", "make_gradient", "make_divergence",
            "make_vector_gradient", "make_vector_laplace",
-           "make_tensor_divergence", "make_poisson_solver", "make_operator"]
+           "make_tensor_divergence", "make_poisson_solver"]
     
\ No newline at end of file
